/*
    Copyright (C) 2005-2010  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _COLLECTIONS_H_
#define _COLLECTIONS_H_

#define MAX_NNTP_SERVERS    32

typedef struct _par2real_file {
    char real_filename[256];
    int num_blocks_found;
    int num_blocks_missing;
    gboolean blocks_found[65535];           // FIXME: What is the maximum number of possible PAR2 blocks?
} PAR2RealFile;

typedef struct _par2file {
    char filename[256];
    GList *real_files;                      // List containing PAR2RealFile pointers
    int num_blocks_found;
    int num_blocks_missing;
} PAR2File;

typedef struct _par2set {
    unsigned char set_id[16];
    guint64 blocksize;
    GList *par2_files;                      // List containing PAR2File pointers
    int total_num_blocks_found;
    int total_num_blocks_missing;
    int total_recovery_blocks_available;
} PAR2Set;

typedef struct _nntp_part {
    const char message_id[256];
    int size;
    int part_num;
    gboolean downloaded;
    gboolean now_downloading;
    unsigned int servers_already_tried;  /* This is actually a bit-masked variable where every server has one bit */
} NNTPPart;

typedef struct _nntp_file {
    volatile int refcount;
    const char subject[256];
    const char poster[256];
    time_t stamp;
    guint64 file_size;
    guint64 file_size_remaining;
    int position;
    int num_parts;
    int num_parts_downloaded;
    int num_parts_failed;
    GList *parts;
    GList *groups;
    gboolean file_is_downloading;
    gboolean file_is_downloaded;
    gboolean stop_flag;
    gboolean now_decoding;
    gboolean now_verifying;
    const char tmp_filename[256];
    const char real_filename[256];
    NGTaskState status;
    NNTPFileType file_type;
    char md5sum[33];
    int par2_start_num;
    int par2_end_num;
    int par2_num_blocks;
    guint64  file_id;               // Necessary for the SQLite database
    gboolean file_is_new;           // Necessary for the SQLite database
    gboolean file_is_changed;       // Necessary for the SQLite database
    gboolean position_is_changed;   // Necessary for the SQLite database
} NNTPFile;

typedef struct _nntp_collection {
    volatile int refcount;
    GStaticMutex mutex;
    const char collection_name[256];
    gboolean now_repairing;
    gboolean now_unpacking;
    char poster[256];
    char groups[256];
    guint64 total_size;
    guint64 total_size_remaining;
    GList *files;
    GList *files_to_download;
    gboolean stop_flag;
    int position;
    GList *par2sets;                // This list contains PAR2Set pointers
    guint64  collection_id;         // Necessary for the SQLite database
    gboolean collection_is_new;     // Necessary for the SQLite database
    gboolean collection_is_changed; // Necessary for the SQLite database
} NNTPCollection;

#endif /* _COLLECTIONS_H */
