/*
    Copyright (C) 2005-2010  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gui.h"

#if defined(HAVE_POLKIT) || defined(WIN32) || defined(DARWIN)
static gboolean autoshutdown_enabled = FALSE;

gboolean        perform_shutdown(void);

static void
tool_button_clicked(GtkToolItem *item, gpointer data)
{
    if (autoshutdown_enabled) {
        autoshutdown_enabled = FALSE;
        //gtk_tool_button_set_label(GTK_TOOL_BUTTON(item), _("Enable automatic shutdown"));
    } else {
        autoshutdown_enabled = TRUE;
        //gtk_tool_button_set_label(GTK_TOOL_BUTTON(item), _("Disable automatic shutdown"));
    }
}

static void
all_downloads_completed(NNTPGrabGlue *glue, gpointer data)
{
    g_print("all downloads completed\n");

    if (autoshutdown_enabled) {
        perform_shutdown();
        g_idle_add((GSourceFunc) gtk_main_quit, NULL);
    }
}
#endif

void
autoshutdown_initialize(void)
{
    if (!nntpgrab_glue_get_is_standalone(glue)) {
        return;
    }

#if defined(HAVE_POLKIT) || defined(WIN32) || defined(DARWIN)
#ifndef HAVE_POLKIT1
    GtkWidget *toolbarMain = nntpgrab_gui_base_get_widget("toolbarMain");
    GtkToolItem *item = gtk_toggle_tool_button_new();
    gtk_tool_button_set_label(GTK_TOOL_BUTTON(item), _("Automatic shutdown"));
    gtk_tool_button_set_stock_id(GTK_TOOL_BUTTON(item), "gtk-disconnect");
    gtk_widget_show(GTK_WIDGET(item));

    gtk_toolbar_insert(GTK_TOOLBAR(toolbarMain), item, -1);

    g_signal_connect(item, "clicked", G_CALLBACK(tool_button_clicked), NULL);
    g_signal_connect(glue, "all_downloads_completed", G_CALLBACK(all_downloads_completed), NULL);
#endif
#endif
}

