/*
    Copyright (C) 2005-2010  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gui.h"
#ifdef HAVE_POLKIT
#include <sys/types.h>
#include <unistd.h>
#include <dbus/dbus-glib.h>

#ifdef HAVE_POLKIT_OLD
#include <polkit-gnome/polkit-gnome.h>
#endif

static gboolean obtain_privileges(gpointer data);

#endif

gboolean
perform_shutdown(void)
{
#ifdef HAVE_POLKIT
    time_t start;
    time_t end;
    guint retval = 0;
    gboolean ret;
    DBusGConnection *connection;
    DBusGProxy *proxy;
    GError *error = NULL;
    //const char *service = "org.freedesktop.Hal";
    //const char *path = "/org/freedesktop/Hal/devices/computer";
    //const char *interface = "org.freedesktop.Hal.Device.SystemPowerManagement";

    const char *service = "org.gnome.SessionManager";
    const char *path = "/org/gnome/SessionManager";
    const char *interface = "org.gnome.SessionManager";
    const char *method = "Shutdown";

    connection = dbus_g_bus_get(DBUS_BUS_SESSION, &error);
    if (!connection) {
        g_print("connection = NULL: %s\n", error->message);
        g_error_free(error);
        return FALSE;
    }

    proxy = dbus_g_proxy_new_for_name_owner (connection,
        service,
        path,
        interface,
        &error);

    if (!proxy) {
        g_warning ("proxy = NULL: %s\n", error->message);
        g_error_free(error);
        dbus_g_connection_unref(connection);
        return FALSE;
    }
    if (DBUS_IS_G_PROXY (proxy) == FALSE) {
        g_warning ("not connected\n");
        g_object_unref (proxy);
        dbus_g_connection_unref(connection);
        return FALSE;
    }

    time (&start);
    ret = dbus_g_proxy_call (proxy, method, &error,
        G_TYPE_INVALID,
        G_TYPE_INT, &retval,
        G_TYPE_INVALID);

    g_object_unref (proxy);
    dbus_g_connection_unref(connection);

    if (retval != 0) {
        g_warning ("%s failed in a horrible way!: %s\n", method, error->message);
        g_error_free(error);
        return FALSE;
    }

    if (!ret) {
        static gboolean beenhere = FALSE;
#ifdef HAVE_POLKIT_OLD
        PolKitResult result = POLKIT_RESULT_UNKNOWN;

        if (!polkit_result_from_string_representation(error->message, &result)) {
            g_print("Failure: %s\n", error->message);
            g_error_free(error);
        }

        g_print("result = %i\n", result);
#endif

        g_print(__FILE__ ":%i %s\n", __LINE__, error->message);

        if (!beenhere) {
            beenhere = TRUE;
            obtain_privileges(error->message);
            perform_shutdown();
        }

        return FALSE;
    }

    /* compare the amount of time that has passed - if it's more than 6 hours
     * then the dbus call timed out (dbus-pending-call.c) */
    time (&end);
    if (difftime (start,end) >= 6*60*60*1000) {
        g_print("DBUS call timed out\n");
        return FALSE;
    }

    g_print("Success\n");

    return ret;
#else
    return FALSE;
#endif
}

#ifdef HAVE_POLKIT_OLD
static void
auth_cb(PolKitAction *action, gboolean gained_privilege, GError *error, gpointer data)
{
    polkit_action_debug(action);

    g_print("gained_privilege = %i\n", gained_privilege);
    if (error) {
        g_print("Error: %s\n", error->message);
    }

    if (!gained_privilege) {
        g_print("Unable to gain privilege\n");
        g_print("error = %#x\n", error);
    } else {
        g_print("Privilege gained, now shutting down\n");
        perform_shutdown();
    }

    gtk_main_quit();
}

static gboolean
obtain_privileges(gpointer data)
{
    PolKitAction *polkit_action;
    GError *error = NULL;

    g_print("obtain privileges\n");

    polkit_action = polkit_action_new();
    polkit_action_set_action_id (polkit_action, (const char*) data);

    if (!polkit_gnome_auth_obtain(polkit_action, 0, getpid(), auth_cb, NULL, &error)) {
        g_print("Error occured: %s\n", error->message);
        g_error_free(error);
        return FALSE;
    }

    g_print("Success\n");

    return FALSE;
}
#endif

#ifdef HAVE_POLKIT1
static gboolean
obtain_privileges(gpointer data)
{
    return FALSE;
}
#endif
