/*
    Copyright (C) 2005-2009  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string.h>
#include "gui.h"
#include "config_gui.h"

void    config_servers_initialize();
void    config_folders_update_folders(void);
void    config_folders_initialize();
void    config_par2_initialize();
void    config_notifications_initialize();
void    config_nzbcreator_initialize();
void    config_autoimport_initialize();
void    config_autoimport_update_folders(void);
void    config_plugins_initialize();

static void
on_cat_selection_changed(GtkTreeSelection *selection, gpointer data)
{
    GtkTreeIter iter;
    GtkTreeModel *model = NULL;
    GtkWidget *widget = NULL;
    GtkWidget *notebookConfigCats = nntpgrab_gui_base_get_widget("notebookConfigCats");
    int pagenum;
    char *label = NULL;

    g_return_if_fail(gtk_tree_selection_count_selected_rows(selection) != 0);

    g_assert(gtk_tree_selection_get_selected(selection, &model, &iter));

    gtk_tree_model_get(model, &iter, 0, &label, 1, &widget, -1);

    pagenum = gtk_notebook_page_num(GTK_NOTEBOOK(notebookConfigCats), GTK_WIDGET(widget));
    gtk_notebook_set_current_page(GTK_NOTEBOOK(notebookConfigCats), pagenum);

    if (!strcmp(label, _("Download folders"))) {
        config_folders_update_folders();
    }

    if (!strcmp(label, _("Automatic NZB import"))) {
        config_autoimport_update_folders();
    }

    g_free(label);
}

void
config_initialize()
{
    GtkWidget *treeConfigCats = nntpgrab_gui_base_get_widget("treeConfigCats");
    GtkTreeSelection *selection;
    GtkListStore *store;
    GtkCellRenderer *cellRenderer;
    GtkTreeIter iter;
    GtkWidget *vboxServers;
    GtkWidget *hboxFolders;
#ifdef HAVE_LIBNOTIFY
    GtkWidget *vboxNotifications;
#endif
    GtkWidget *vboxPAR2;
    GtkWidget *vboxDisplay;
#if 0
    GtkWidget *vboxNZBCreator;
#endif
    GtkWidget *vboxAutoImport;
    GtkWidget *hboxPlugins;

    config_servers_initialize();
    config_folders_initialize();
    config_par2_initialize();
    config_notifications_initialize();
    config_nzbcreator_initialize();
#if GLIB_CHECK_VERSION(2,16,0)
    config_autoimport_initialize();
#endif
    config_plugins_initialize();

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeConfigCats));
    gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);

    cellRenderer = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(treeConfigCats), -1, _("Category"), cellRenderer, "text", 0, NULL);

    store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_OBJECT);

    gtk_tree_view_set_model(GTK_TREE_VIEW(treeConfigCats), GTK_TREE_MODEL(store));

    gtk_tree_view_set_enable_search(GTK_TREE_VIEW(treeConfigCats), FALSE);

    g_signal_connect(selection, "changed", G_CALLBACK(on_cat_selection_changed), NULL);

    vboxServers = nntpgrab_gui_base_get_widget("vboxServers");
    hboxFolders = nntpgrab_gui_base_get_widget("hboxFolders");
#ifdef HAVE_LIBNOTIFY
    vboxNotifications = nntpgrab_gui_base_get_widget("vboxNotifications");
#endif
    vboxPAR2 = nntpgrab_gui_base_get_widget("vboxPAR2");
    vboxDisplay = nntpgrab_gui_base_get_widget("vboxDisplay");
#if 0
    vboxNZBCreator = nntpgrab_gui_base_get_widget("vboxNZBCreator");
#endif
#if GLIB_CHECK_VERSION(2,16,0)
    vboxAutoImport = nntpgrab_gui_base_get_widget("vboxAutoImport");
#endif
    hboxPlugins = nntpgrab_gui_base_get_widget("hboxPlugins");

    gtk_list_store_clear(store);

    gtk_list_store_append(store, &iter);
    gtk_list_store_set(store, &iter, 0, _("Servers"), 1, vboxServers, -1);
    gtk_tree_selection_select_iter(selection, &iter);

    gtk_list_store_append(store, &iter);
    gtk_list_store_set(store, &iter, 0, _("Download folders"), 1, hboxFolders, -1);

#ifdef HAVE_LIBNOTIFY
    gtk_list_store_append(store, &iter);
    gtk_list_store_set(store, &iter, 0, _("Notifications"), 1, vboxNotifications, -1);
#endif

    gtk_list_store_append(store, &iter);
    gtk_list_store_set(store, &iter, 0, _("PAR2 handling"), 1, vboxPAR2, -1);

    gtk_list_store_append(store, &iter);
    gtk_list_store_set(store, &iter, 0, _("Display settings"), 1, vboxDisplay, -1);

#if 0
    gtk_list_store_append(store, &iter);
    gtk_list_store_set(store, &iter, 0, _("Online Search"), 1, vboxNZBCreator, -1);
#endif

#if GLIB_CHECK_VERSION(2,16,0)
/* GDirectoryMonitor isn't available yet for OSX */
#ifndef __APPLE__
    gtk_list_store_append(store, &iter);
    gtk_list_store_set(store, &iter, 0, _("Automatic NZB import"), 1, vboxAutoImport, -1);
#endif
#endif

    gtk_list_store_append(store, &iter);
    gtk_list_store_set(store, &iter, 0, _("Plugins"), 1, hboxPlugins, -1);
}

