/*
    Copyright (C) 2005-2010  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gui.h"

G_MODULE_EXPORT void
on_chkEnableWebserver_toggled(GtkToggleButton *caller, gpointer data)
{
    ConfigOpts opts = nntpgrab_config_get_opts(glue);
    gboolean state = gtk_toggle_button_get_active(caller);

    if (opts.enable_webserver != state) {
        opts.enable_webserver = state;
    }

    nntpgrab_config_set_opts(glue, opts);
}

G_MODULE_EXPORT void
on_btnConfigWebserverPortApply_clicked(GtkButton *caller, gpointer data)
{
    ConfigOpts opts = nntpgrab_config_get_opts(glue);
    GtkWidget *spinConfigWebserverPort = nntpgrab_gui_base_get_widget("spinConfigWebserverPort");
    int new_value = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinConfigWebserverPort));

    if (new_value != opts.webserver_port) {
        opts.webserver_port = new_value;
        nntpgrab_config_set_opts(glue, opts);
    }
}

G_MODULE_EXPORT void
on_spinConfigWebserverPort_value_changed(GtkSpinButton *caller, gpointer data)
{
    GtkWidget *btnConfigWebserverPortApply = nntpgrab_gui_base_get_widget("btnConfigWebserverPortApply");

    gtk_widget_set_sensitive(btnConfigWebserverPortApply, TRUE);
}

G_MODULE_EXPORT void
on_chkEnableLogger_toggled(GtkToggleButton *caller, gpointer data)
{
    ConfigOpts opts = nntpgrab_config_get_opts(glue);
    gboolean state = gtk_toggle_button_get_active(caller);

    if (opts.enable_logger != state) {
        opts.enable_logger = state;
    }

    nntpgrab_config_set_opts(glue, opts);
}

static void
config_changed(NNTPGrabGlue *glue, gpointer data)
{
    GtkWidget *chkEnableWebserver = nntpgrab_gui_base_get_widget("chkEnableWebserver");
    GtkWidget *spinConfigWebserverPort = nntpgrab_gui_base_get_widget("spinConfigWebserverPort");
    GtkWidget *btnConfigWebserverPortApply = nntpgrab_gui_base_get_widget("btnConfigWebserverPortApply");
    GtkWidget *labelConfigWebserverPort = nntpgrab_gui_base_get_widget("labelConfigWebserverPort");
    GtkWidget *chkEnableLogger = nntpgrab_gui_base_get_widget("chkEnableLogger");
    ConfigOpts opts = nntpgrab_config_get_opts(glue);

    g_signal_handlers_block_by_func (G_OBJECT (chkEnableWebserver), G_CALLBACK (on_chkEnableWebserver_toggled), NULL);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkEnableWebserver), opts.enable_webserver);
    g_signal_handlers_unblock_by_func (G_OBJECT (chkEnableWebserver), G_CALLBACK (on_chkEnableWebserver_toggled), NULL);

    g_signal_handlers_block_by_func (G_OBJECT (chkEnableLogger), G_CALLBACK (on_chkEnableLogger_toggled), NULL);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkEnableLogger), opts.enable_logger);
    g_signal_handlers_unblock_by_func (G_OBJECT (chkEnableLogger), G_CALLBACK (on_chkEnableLogger_toggled), NULL);

    if (!opts.enable_webserver) {
        gtk_widget_hide(spinConfigWebserverPort);
        gtk_widget_hide(labelConfigWebserverPort);
        gtk_widget_hide(btnConfigWebserverPortApply);
    } else {
        gtk_widget_show(spinConfigWebserverPort);
        gtk_widget_show(labelConfigWebserverPort);
        gtk_widget_show(btnConfigWebserverPortApply);

        gtk_widget_set_sensitive(btnConfigWebserverPortApply, FALSE);
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinConfigWebserverPort), opts.webserver_port);
    }
}

void
config_plugins_initialize()
{
    ng_signal_connect(glue, "config_changed", NG_CALLBACK(config_changed), NULL);
    config_changed(glue, NULL);
}
