/*
    Copyright (C) 2005-2010  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string.h>
#include "gui.h"

static void update_avail_servers(void);
static char *get_selected_servername(void);
static void build_config_window(ConfigServer config);

// If strlen(original_servername) > 0 then we are editing a already existing server
static char original_servername[128];

G_MODULE_EXPORT void
on_btnAddUsenetServer_clicked(GtkWidget *caller, gpointer data)
{
    ConfigServer config;

    memset(&config, 0, sizeof(config));

    config.port = 119;
    config.max_threads = 3;
    config.priority = 1;
    config.enabled = TRUE;

    build_config_window(config);

    memset(&original_servername, 0, sizeof(original_servername));
}

G_MODULE_EXPORT void
on_btnRemoveUsenetServer_clicked(GtkWidget *caller, gpointer data)
{
    char *servername;
    char *errmsg = NULL;

    servername = get_selected_servername();
    if (!nntpgrab_config_del_server(glue, servername, &errmsg)) {
        g_warning("%s", errmsg);
        g_free(errmsg);
    }

    g_free(servername);

    update_avail_servers();
}

G_MODULE_EXPORT void
on_btnEditUsenetServer_clicked(GtkWidget *caller, gpointer data)
{
    ConfigServer config;
    char *servername;

    servername = get_selected_servername();

    g_assert(nntpgrab_config_get_server_info(glue, servername, &config));

    g_free(servername);

    build_config_window(config);

    strncpy(original_servername, config.servername, sizeof(original_servername));
}

G_MODULE_EXPORT void
on_treeConfigServers_row_activated(GtkWidget *caller, GtkTreePath *path, GtkTreeViewColumn *column, gpointer data)
{
    on_btnEditUsenetServer_clicked(caller, data);
}

static char *
get_selected_servername(void)
{
    GtkWidget *treeConfigServers = nntpgrab_gui_base_get_widget("treeConfigServers");
    GtkTreeModel *model;
    GtkTreeSelection *selection;
    GtkTreeIter iter;
    char *servername = NULL;

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeConfigServers));
    g_assert(gtk_tree_selection_get_selected(selection, &model, &iter));

    gtk_tree_model_get(model, &iter, 0, &servername, -1);

    return servername;
}

G_MODULE_EXPORT void
on_btnConfigSave_clicked(GtkWidget *caller, gpointer data)
{
    GtkWidget *windowConfig;
    ConfigServer config;
    GtkWidget *txtConfigName;
    GtkWidget *txtConfigHostname;
    GtkWidget *chkConfigServerRequiresAuthentication;
    GtkWidget *txtConfigUsername;
    GtkWidget *txtConfigPassword;
    GtkWidget *spinConfigPort;
    GtkWidget *spinConfigMaxThreads;
    GtkWidget *chkUseSSL;
    GtkWidget *comboConfigServerPriority;
    GtkWidget *chkServerIsEnabled;
    char *errmsg = NULL;

    windowConfig = nntpgrab_gui_base_get_widget("windowConfig");
    txtConfigName = nntpgrab_gui_base_get_widget("txtConfigName");
    txtConfigHostname = nntpgrab_gui_base_get_widget("txtConfigHostname");
    chkConfigServerRequiresAuthentication = nntpgrab_gui_base_get_widget("chkConfigServerRequiresAuthentication");
    txtConfigUsername = nntpgrab_gui_base_get_widget("txtConfigUsername");
    txtConfigPassword = nntpgrab_gui_base_get_widget("txtConfigPassword");
    spinConfigPort = nntpgrab_gui_base_get_widget("spinConfigPort");
    spinConfigMaxThreads = nntpgrab_gui_base_get_widget("spinConfigMaxThreads");
    chkUseSSL = nntpgrab_gui_base_get_widget("chkUseSSL");
    comboConfigServerPriority = nntpgrab_gui_base_get_widget("comboConfigServerPriority");
    chkServerIsEnabled = nntpgrab_gui_base_get_widget("chkServerIsEnabled");

    memset(&config, 0, sizeof(config));

    strncpy(config.servername, gtk_entry_get_text(GTK_ENTRY(txtConfigName)), sizeof(config.servername));
    strncpy(config.hostname, gtk_entry_get_text(GTK_ENTRY(txtConfigHostname)), sizeof(config.hostname));
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chkConfigServerRequiresAuthentication))) {
        strncpy(config.username, gtk_entry_get_text(GTK_ENTRY(txtConfigUsername)), sizeof(config.username));
        strncpy(config.password, gtk_entry_get_text(GTK_ENTRY(txtConfigPassword)), sizeof(config.password));
    }
    config.port = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinConfigPort));
    config.max_threads = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinConfigMaxThreads));
    config.use_ssl = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chkUseSSL));
    config.enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chkServerIsEnabled));
    config.priority = gtk_combo_box_get_active(GTK_COMBO_BOX(comboConfigServerPriority));
    if (config.priority == -1) {
        config.priority = SERVER_PRIORITY_NORMAL;
    }

    // verify the input
    if (strlen(config.servername) == 0) {
        nntpgrab_gui_base_dialog_show(windowConfig, _("Please enter a servername"), GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
        return;
    }

    if (strlen(config.hostname) == 0) {
        nntpgrab_gui_base_dialog_show(windowConfig, _("Please enter a hostname"), GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
        return;
    }

    if (strlen(original_servername) > 0) {
        if (!nntpgrab_config_edit_server(glue, original_servername, config, &errmsg)) {
            char *msg = g_strdup_printf(_("Unable to edit server:\n%s"), errmsg);
            nntpgrab_gui_base_dialog_show(windowConfig, msg, GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
            g_free(msg);
            g_free(errmsg);
        }
    } else {
        if (!nntpgrab_config_add_server(glue, config, &errmsg)) {
            char *msg = g_strdup_printf(_("Unable to add server:\n%s"), errmsg);
            nntpgrab_gui_base_dialog_show(windowConfig, msg, GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
            g_free(msg);
            g_free(errmsg);
            return;
        }
    }

    gtk_widget_hide(windowConfig);

    update_avail_servers();
}

G_MODULE_EXPORT void
on_btnConfigCancel_clicked(GtkWidget *caller, gpointer data)
{
    GtkWidget *windowConfig = nntpgrab_gui_base_get_widget("windowConfig");

    gtk_widget_hide(windowConfig);
}

G_MODULE_EXPORT gboolean
on_windowConfig_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
    int retval = nntpgrab_gui_base_dialog_show(widget, _("Do you want to save changes?"), GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO | GTK_BUTTONS_CANCEL);

    switch (retval) {
        case GTK_RESPONSE_CANCEL:
            break;

        case GTK_RESPONSE_YES:
            on_btnConfigSave_clicked(widget, user_data);
            break;

        case GTK_RESPONSE_NO:
            on_btnConfigCancel_clicked(widget, user_data);
            break;

        default:
            g_error(_("Invalid return value: %i\n"), retval);
    }

    return TRUE;
}

G_MODULE_EXPORT void
on_chkConfigServerRequiresAuthentication_toggled(GtkToggleButton *caller, gpointer data)
{
    GtkWidget *frameConfigAuthentication = nntpgrab_gui_base_get_widget("frameConfigAuthentication");

    if (gtk_toggle_button_get_active(caller)) {
        gtk_widget_set_sensitive(frameConfigAuthentication, TRUE);
    } else {
        gtk_widget_set_sensitive(frameConfigAuthentication, FALSE);
    }
}

G_MODULE_EXPORT void
on_chkConfigServersEnableShaping_toggled(GtkToggleButton *caller, gpointer data)
{
    GtkWidget *spinConfigServerMaxBandwidth = nntpgrab_gui_base_get_widget("spinConfigServerMaxBandwidth");
    GtkWidget *labelConfigServerMaxBandwidth = nntpgrab_gui_base_get_widget("labelConfigServerMaxBandwidth");
    GtkWidget *btnConfigServerMaxBandwidthApply = nntpgrab_gui_base_get_widget("btnConfigServerMaxBandwidthApply");

    ConfigOpts opts = nntpgrab_config_get_opts(glue);

    if (gtk_toggle_button_get_active(caller)) {
        gtk_widget_show(spinConfigServerMaxBandwidth);
        gtk_widget_show(labelConfigServerMaxBandwidth);
        gtk_widget_show(btnConfigServerMaxBandwidthApply);

        if (!opts.enable_bandwidth_shaping) {
            opts.enable_bandwidth_shaping = TRUE;
            nntpgrab_config_set_opts(glue, opts);

            gtk_widget_set_sensitive(btnConfigServerMaxBandwidthApply, FALSE);
        }

        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinConfigServerMaxBandwidth), opts.max_bandwidth);
    } else {
        gtk_widget_hide(spinConfigServerMaxBandwidth);
        gtk_widget_hide(labelConfigServerMaxBandwidth);
        gtk_widget_hide(btnConfigServerMaxBandwidthApply);

        if (opts.enable_bandwidth_shaping) {
            opts.enable_bandwidth_shaping = FALSE;
            nntpgrab_config_set_opts(glue, opts);
        }
    }
}

G_MODULE_EXPORT void
on_spinConfigServerMaxBandwidth_value_changed(GtkSpinButton *caller, gpointer data)
{
    GtkWidget *btnConfigServerMaxBandwidthApply = nntpgrab_gui_base_get_widget("btnConfigServerMaxBandwidthApply");

    gtk_widget_set_sensitive(btnConfigServerMaxBandwidthApply, TRUE);
}

G_MODULE_EXPORT void
on_btnConfigServerMaxBandwidthApply_clicked(GtkButton *caller, gpointer data)
{
    GtkWidget *spinConfigServerMaxBandwidth = nntpgrab_gui_base_get_widget("spinConfigServerMaxBandwidth");
    ConfigOpts opts = nntpgrab_config_get_opts(glue);

    opts.max_bandwidth = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinConfigServerMaxBandwidth));

    nntpgrab_config_set_opts(glue, opts);

    gtk_widget_set_sensitive(GTK_WIDGET(caller), FALSE);
}

static void
on_selection_changed(GtkTreeSelection *selection, gpointer data)
{
    GtkWidget *btnRemoveUsenetServer;
    GtkWidget *btnEditUsenetServer;

    btnRemoveUsenetServer = nntpgrab_gui_base_get_widget("btnRemoveUsenetServer");
    btnEditUsenetServer = nntpgrab_gui_base_get_widget("btnEditUsenetServer");

    if (gtk_tree_selection_count_selected_rows(selection) == 0) {
        gtk_widget_set_sensitive(btnRemoveUsenetServer, FALSE);
        gtk_widget_set_sensitive(btnEditUsenetServer, FALSE);
    } else {
        gtk_widget_set_sensitive(btnRemoveUsenetServer, TRUE);
        gtk_widget_set_sensitive(btnEditUsenetServer, TRUE);
    }
}

static void
build_config_window(ConfigServer config)
{
    GtkWidget *windowConfig;
    GtkWidget *txtConfigName;
    GtkWidget *txtConfigHostname;
    GtkWidget *chkConfigServerRequiresAuthentication;
    GtkWidget *txtConfigUsername;
    GtkWidget *txtConfigPassword;
    GtkWidget *spinConfigPort;
    GtkWidget *spinConfigMaxThreads;
    GtkWidget *chkUseSSL;
    GtkWidget *comboConfigServerPriority;
    GtkWidget *chkServerIsEnabled;

    windowConfig = nntpgrab_gui_base_get_widget("windowConfig");
    txtConfigName = nntpgrab_gui_base_get_widget("txtConfigName");
    txtConfigHostname = nntpgrab_gui_base_get_widget("txtConfigHostname");
    chkConfigServerRequiresAuthentication = nntpgrab_gui_base_get_widget("chkConfigServerRequiresAuthentication");
    txtConfigUsername = nntpgrab_gui_base_get_widget("txtConfigUsername");
    txtConfigPassword = nntpgrab_gui_base_get_widget("txtConfigPassword");
    spinConfigPort = nntpgrab_gui_base_get_widget("spinConfigPort");
    spinConfigMaxThreads = nntpgrab_gui_base_get_widget("spinConfigMaxThreads");
    chkUseSSL = nntpgrab_gui_base_get_widget("chkUseSSL");
    comboConfigServerPriority = nntpgrab_gui_base_get_widget("comboConfigServerPriority");
    chkServerIsEnabled = nntpgrab_gui_base_get_widget("chkServerIsEnabled");

    gtk_entry_set_text(GTK_ENTRY(txtConfigName), config.servername);
    gtk_entry_set_text(GTK_ENTRY(txtConfigHostname), config.hostname);
    if (strlen(config.username) > 0) {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkConfigServerRequiresAuthentication), TRUE);
        gtk_entry_set_text(GTK_ENTRY(txtConfigUsername), config.username);
        gtk_entry_set_text(GTK_ENTRY(txtConfigPassword), config.password);
    } else {
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkConfigServerRequiresAuthentication), FALSE);
        gtk_entry_set_text(GTK_ENTRY(txtConfigUsername), "");
        gtk_entry_set_text(GTK_ENTRY(txtConfigPassword), "");
    }

    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinConfigPort), config.port);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinConfigMaxThreads), config.max_threads);
    gtk_combo_box_set_active(GTK_COMBO_BOX(comboConfigServerPriority), config.priority);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkUseSSL), config.use_ssl);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkServerIsEnabled), config.enabled);

    gtk_widget_show(windowConfig);
}

static void
update_avail_servers(void)
{
    GtkWidget *treeConfigServers;
    GtkTreeModel *model;
    NGList *list, *servers;
    GtkTreeIter iter;

    treeConfigServers = nntpgrab_gui_base_get_widget("treeConfigServers");
    model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeConfigServers));

    gtk_list_store_clear(GTK_LIST_STORE(model));

    servers = nntpgrab_config_get_avail_servers(glue);

    list = servers;
    while (list) {
        const char *servername = (const char *) list->data;
        ConfigServer config;
        char *prio;

        g_return_if_fail(nntpgrab_config_get_server_info(glue, servername, &config));

        switch(config.priority) {
            case SERVER_PRIORITY_HIGH:
                prio = _("High");
                break;

            case SERVER_PRIORITY_LOW:
                prio = _("Low");
                break;

            case SERVER_PRIORITY_NORMAL:
            default:
                prio = _("Normal");
                break;
        }

        gtk_list_store_append(GTK_LIST_STORE(model), &iter);
        gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, config.servername, 1, prio, 2, config.max_threads, 3, config.enabled, -1);

        list = ng_list_next(list);
    }

    if (gtk_tree_model_get_iter_first(model, &iter)) {
        GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeConfigServers));
        gtk_tree_selection_select_iter(selection, &iter);
    }

    nntpgrab_config_free_avail_servers(glue, servers);
}

static void
config_changed(NNTPGrabGlue *glue, gpointer data)
{
    GtkWidget *chkConfigServersEnableShaping = nntpgrab_gui_base_get_widget("chkConfigServersEnableShaping");
    GtkWidget *spinConfigServerMaxBandwidth = nntpgrab_gui_base_get_widget("spinConfigServerMaxBandwidth");
    GtkWidget *labelConfigServerMaxBandwidth = nntpgrab_gui_base_get_widget("labelConfigServerMaxBandwidth");
    GtkWidget *btnConfigServerMaxBandwidthApply = nntpgrab_gui_base_get_widget("btnConfigServerMaxBandwidthApply");

    ConfigOpts opts = nntpgrab_config_get_opts(glue);

    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chkConfigServersEnableShaping)) && !opts.enable_bandwidth_shaping) {
        /* Somebody else disabled bandwidth shaping */
        gtk_widget_hide(spinConfigServerMaxBandwidth);
        gtk_widget_hide(labelConfigServerMaxBandwidth);
        gtk_widget_hide(btnConfigServerMaxBandwidthApply);
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkConfigServersEnableShaping), FALSE);
    } else if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chkConfigServersEnableShaping)) && opts.enable_bandwidth_shaping) {
        /* Somebody else enabled bandwidth shaping */
        gtk_widget_show(spinConfigServerMaxBandwidth);
        gtk_widget_show(labelConfigServerMaxBandwidth);
        gtk_widget_show(btnConfigServerMaxBandwidthApply);
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkConfigServersEnableShaping), TRUE);
        gtk_widget_set_sensitive(btnConfigServerMaxBandwidthApply, FALSE);
    }

    /* Always update the spinner value when bandwidth shaping is active */
    if (opts.enable_bandwidth_shaping) {
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinConfigServerMaxBandwidth), opts.max_bandwidth);
    }

    update_avail_servers();
}

static void
server_check_toggled(GtkWidget *caller, gchar *path, gpointer data)
{
    gboolean value;
    GtkTreeIter iter;
    GtkWidget *treeConfigServers;
    GtkTreeModel *model;
    char *errmsg = NULL;
    char *servername = NULL;
    ConfigServer config;

    treeConfigServers = nntpgrab_gui_base_get_widget("treeConfigServers");
    model = gtk_tree_view_get_model(GTK_TREE_VIEW(treeConfigServers));

    if (!gtk_tree_model_get_iter_from_string(model, &iter, path)) {
        return;
    }

    gtk_tree_model_get(model, &iter, 0, &servername, 3, &value, -1);
    if (value) {
        value = FALSE;
    } else {
        value = TRUE;
    }

    /* Change the status in the NNTPGrab Core */
    if (!nntpgrab_config_get_server_info(glue, servername, &config)) {
        return;
    }

    config.enabled = value;

    /* Temporary disable the signal handler while the change is applied */
    g_signal_handlers_block_by_func (G_OBJECT (caller), G_CALLBACK (server_check_toggled), NULL);

    if (!nntpgrab_config_edit_server(glue, servername, config, &errmsg)) {
        GtkWidget *windowMain = nntpgrab_gui_base_get_widget("windowMain");
        char *msg = g_strdup_printf(_("Unable to edit server:\n%s"), errmsg);
        nntpgrab_gui_base_dialog_show(windowMain, msg, GTK_MESSAGE_INFO, GTK_BUTTONS_OK);
        g_free(msg);
        g_free(errmsg);
        g_free(servername);

        g_signal_handlers_unblock_by_func (G_OBJECT (caller), G_CALLBACK (server_check_toggled), NULL);
        return;
    }

    g_free(servername);

    gtk_list_store_set(GTK_LIST_STORE(model), &iter, 3, value, -1);

    g_signal_handlers_unblock_by_func (G_OBJECT (caller), G_CALLBACK (server_check_toggled), NULL);
}

void
config_servers_initialize()
{
    GtkWidget *treeConfigServers = nntpgrab_gui_base_get_widget("treeConfigServers");
    GtkTreeSelection *selection;
    GtkListStore *store;
    GtkCellRenderer *cellRenderer;
    GtkTreeViewColumn *column;

    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeConfigServers));
    gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);

    cellRenderer = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(treeConfigServers), -1, _("Servername"), cellRenderer, "text", 0, NULL);
    column = gtk_tree_view_get_column(GTK_TREE_VIEW(treeConfigServers), 0);
    gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
    gtk_tree_view_column_set_min_width(column, 200);

    cellRenderer = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(treeConfigServers), -1, _("Priority"), cellRenderer, "text", 1, NULL);
    column = gtk_tree_view_get_column(GTK_TREE_VIEW(treeConfigServers), 1);
    gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
    gtk_tree_view_column_set_fixed_width(column, 75);

    cellRenderer = gtk_cell_renderer_text_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(treeConfigServers), -1, _("Max threads"), cellRenderer, "text", 2, NULL);
    column = gtk_tree_view_get_column(GTK_TREE_VIEW(treeConfigServers), 2);
    gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
    gtk_tree_view_column_set_fixed_width(column, 150);

    cellRenderer = gtk_cell_renderer_toggle_new();
    gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(treeConfigServers), -1, _("Enabled"), cellRenderer, "active", 3, NULL);
    g_signal_connect((gpointer) cellRenderer, "toggled", G_CALLBACK(server_check_toggled), NULL);
    column = gtk_tree_view_get_column(GTK_TREE_VIEW(treeConfigServers), 3);
    gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
    gtk_tree_view_column_set_fixed_width(column, 125);

    store = gtk_list_store_new(4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_BOOLEAN);
    gtk_tree_view_set_model(GTK_TREE_VIEW(treeConfigServers), GTK_TREE_MODEL(store));

    config_changed(glue, NULL);

    g_signal_connect(selection, "changed", G_CALLBACK(on_selection_changed), NULL);
    on_selection_changed(selection, NULL);

    ng_signal_connect(glue, "config_changed", NG_CALLBACK(config_changed), NULL);
}
