#include "DialogManagePlugins.h"
#include "ui_DialogManagePlugins.h"

DialogManagePlugins::DialogManagePlugins(QNNTPGrabGlue *glue, QWidget *parent) :
QDialog(parent),
ui(new Ui::DialogManagePlugins)
{
    this->glue = glue;
    ui->setupUi(this);

    connect(glue, SIGNAL(pluginLoaded(QString,bool)), SLOT(onpluginLoaded(QString,bool)));
    connect(glue, SIGNAL(pluginUnloaded(QString)), SLOT(onpluginUnloaded(QString)));

    QList<QString> plugins = this->glue->getAvailPlugins();
    bool first = true;
    int i ;
    for(i = 0; i < plugins.length(); i++)
    {
        NNTPGrabPluginInfo plugin_info;
        if(!glue->getPluginInfo(plugins[i], &plugin_info))
        {
            qWarning("Unable to find a plugin named '%s'", plugins[i].toUtf8().data());
            continue;
        }

        QTreeWidgetItem *plugin_item = new QTreeWidgetItem();
        if(plugin_info.is_loaded)
        {
            plugin_item->setCheckState(0, Qt::Checked);
        }
        else
        {
            plugin_item->setCheckState(0, Qt::Unchecked);
        }

        if(plugin_info.is_persistent)
        {
            plugin_item->setCheckState(1, Qt::Checked);
        }
        else
        {
            plugin_item->setCheckState(1, Qt::Unchecked);
        }
        plugin_item->setText(2, plugin_info.name);

        ui->treeWidget->addTopLevelItem(plugin_item);
        if(first)
        {
            ui->treeWidget->setItemSelected(plugin_item, true);
            first = false;
        }
    }
}

DialogManagePlugins::~DialogManagePlugins()
{
    delete ui;
}

void DialogManagePlugins::onpluginLoaded(QString plugin_name, bool is_persistent)
{
    qDebug("plugin loaded");

    /*plugin_item->setCheckState(0, Qt::Checked);

    if(is_persistent)
    {
        plugin_item->setCheckState(1, Qt::Checked);
    }
    else
    {
        plugin_item->setCheckState(1, Qt::Unchecked);
    }*/
}


void DialogManagePlugins::onpluginUnloaded(QString plugin_name)
{
    qDebug("plugin unloaded");

    //plugin_item->setCheckState(0, Qt::Unchecked);
}

void DialogManagePlugins::on_treeWidget_itemSelectionChanged()
{
    if(ui->treeWidget->selectedItems().count() != 1)
        return;

    QTreeWidgetItem *item = ui->treeWidget->selectedItems()[0];
    NNTPGrabPluginInfo plugin_info;
    if(!glue->getPluginInfo(item->text(2), &plugin_info))
    {
        qWarning("Unable to get plugin info for plugin: %s",item->text(2).toUtf8().data());
        return;
    }

    ui->lb_plugin_name->setText(plugin_info.name);
    ui->lb_plugin_author->setText(plugin_info.author);
    ui->lb_plugin_version->setText(plugin_info.version);
    ui->lb_plugin_url->setText(plugin_info.url);
    ui->lb_plugin_description->setText(plugin_info.description);
}

void DialogManagePlugins::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
