#include <QMetaType>
#ifdef WIN32
#include <windows.h>
#endif

#include <stdlib.h>
#include <string.h>

#include "QNNTPGrabGlue.h"
#include "nntpgrab_glue.h"
#include "nntpgrab_utils.h"

QNNTPGrabGlueThread::QNNTPGrabGlueThread(NNTPGrabGlue *glue)
{
    this->glue = glue;
}

QNNTPGrabGlueThread::~QNNTPGrabGlueThread()
{
    if (this->isRunning()) {
        ng_event_handler_loop_quit();
        this->wait();
    }
}

void QNNTPGrabGlueThread::run()
{
    ng_event_handler_loop_run();
}

QNNTPGrabGlue::QNNTPGrabGlue()
{
    glue = NULL;
    qRegisterMetaType<QNNTPGrabGlue::NNTPDisconnectType>("QNNTPGrabGlue::NNTPDisconnectType");
    qRegisterMetaType<QNNTPGrabGlue::TaskState>("QNNTPGrabGlue::TaskState");
    qRegisterMetaType<QNNTPGrabGlue::LogLevel>("QNNTPGrabGlue::LogLevel");
    qRegisterMetaType<QNNTPGrabGlue::SchedularState>("QNNTPGrabGlue::SchedularState");
    qRegisterMetaType<QList<QString> >("QList<QString>");
    qRegisterMetaType<time_t>("time_t");
}

QNNTPGrabGlue::~QNNTPGrabGlue()
{
    if (glue) {
        nntpgrab_glue_destroy(glue);
        glue = NULL;
    }
}

/*******************************************
 * Callback wrappers                       *
 *******************************************/
void QNNTPGrabGlue::configChangedWrapper(NNTPGrabGlue *glue, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->configChanged();
}

void QNNTPGrabGlue::allDownloadsCompletedWrapper(NNTPGrabGlue *glue, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->allDownloadsCompleted();
}

void QNNTPGrabGlue::partDownloadStartWrapper(NNTPGrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->partDownloadStart(QString(servername), conn_id, QString(collection_name), QString(subject), part_num);
}

void QNNTPGrabGlue::partDoneWrapper(NNTPGrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, int size, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->partDone(QString(servername), conn_id, QString(collection_name), QString(subject), part_num, size);
}

void QNNTPGrabGlue::partFailedWrapper(NNTPGrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num,int size, bool all_servers_tried, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->partFailed(QString(servername), conn_id, QString(collection_name), QString(subject), part_num, size, all_servers_tried);
}

void QNNTPGrabGlue::partProgressUpdateWrapper(NNTPGrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, int bytes_downloaded, int bytes_total, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->partProgressUpdate(QString(servername), conn_id, QString(collection_name), QString(subject), part_num, bytes_downloaded, bytes_total);
}

void QNNTPGrabGlue::trafficMonitorUpdateWrapper(NNTPGrabGlue *glue, int bytes_received1, int bytes_received2, int bytes_received3, int bytes_received4, int bytes_received5, int bytes_received6, int bytes_received7, int bytes_received8, int bytes_received9, int bytes_received10, nguint64 stamp, double average, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->trafficMonitorUpdate(bytes_received1, bytes_received2, bytes_received3, bytes_received4, bytes_received5, bytes_received6,
                                                       bytes_received7, bytes_received8, bytes_received9, bytes_received10, stamp, average);
}

void QNNTPGrabGlue::collectionAddedWrapper(NNTPGrabGlue *glue, const char *collection_name, const char *poster, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->collectionAdded(QString(collection_name), QString(poster));
}

void QNNTPGrabGlue::collectionModifiedWrapper(NNTPGrabGlue *glue, const char *collection_name, const char *poster, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->collectionModified(QString(collection_name), QString(poster));
}

void QNNTPGrabGlue::collectionRemovedWrapper(NNTPGrabGlue *glue, const char *collection_name, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->collectionRemoved(QString(collection_name));
}

void QNNTPGrabGlue::fileAddedWrapper(NNTPGrabGlue *glue, const char *collection_name, const char *subject, const char *poster, nguint64 stamp, nguint64 file_size, nguint64 total_size, nguint64 total_size_remaining, TaskState state, int num_parts, NGList *groups, void *data)
{
    Q_UNUSED(glue);

    QList<QString> list_groups;
    NGList *list;

    list = groups;
    while (list) {
        list_groups.append(QString((const char *) list->data));
        list = ng_list_next(list);
    }

    emit ((QNNTPGrabGlue*) data)->fileAdded(QString(collection_name), QString(subject), QString(poster), (time_t) stamp, file_size, total_size, total_size_remaining, state, num_parts, list_groups);
}

void QNNTPGrabGlue::fileRemovedWrapper(NNTPGrabGlue *glue, const char *collection_name, const char *subject, nguint64 total_size, nguint64 total_size_remaining, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->fileRemoved(QString(collection_name), QString(subject), total_size, total_size_remaining);
}

void QNNTPGrabGlue::fileDownloadStateUpdateWrapper(NNTPGrabGlue *glue, const char *collection_name, const char *subject, int num_parts_total, int num_parts_done, int num_parts_failed, nguint64 file_size, nguint64 file_size_remaining, nguint64 total_size, nguint64 total_size_remaining, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->fileDownloadStateUpdate(QString(collection_name), QString(subject), num_parts_total, num_parts_done, num_parts_failed, file_size, file_size_remaining, total_size, total_size_remaining);
}

void QNNTPGrabGlue::fileStateChangedWrapper(NNTPGrabGlue *glue, const char *collection_name, const char *subject, const char *real_filename, TaskState old_state, TaskState new_state, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->fileStateChanged(QString(collection_name), QString(subject), QString(real_filename), (QNNTPGrabGlue::TaskState) old_state, (QNNTPGrabGlue::TaskState) new_state);
}

void QNNTPGrabGlue::connectionConnectingWrapper(NNTPGrabGlue *glue, const char *servername, int conn_id, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->connectionConnecting(QString(servername), conn_id);
}

void QNNTPGrabGlue::connectionConnectedWrapper(NNTPGrabGlue *glue, const char *servername, int conn_id, const char *welcome_msg, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->connectionConnected(QString(servername), conn_id, welcome_msg);
}

void QNNTPGrabGlue::connectionDisconnectWrapper(NNTPGrabGlue *glue, const char *servername, int conn_id, NNTPDisconnectType disconnect_type, const char *reason, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->connectionDisconnect(QString(servername), conn_id, disconnect_type, QString(reason));
}

void QNNTPGrabGlue::logMessageWrapper(NNTPGrabGlue *glue, const char *component, NGLogLevel log_level, const char *msg, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->logMessage(QString(component), (QNNTPGrabGlue::LogLevel) log_level, QString(msg));
}

void QNNTPGrabGlue::schedularStateChangedWrapper(NNTPGrabGlue *glue, NGSchedularState new_state, const char *reason, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->schedularStateChanged((QNNTPGrabGlue::SchedularState) new_state, QString(reason));
}

void QNNTPGrabGlue::fileMovedWrapper(NNTPGrabGlue *glue, const char *orig_collection_name, const char *subject, const char *new_collection_name, int old_position, int new_position, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->fileMoved(QString(orig_collection_name), QString(subject), QString(new_collection_name), old_position, new_position);
}

void QNNTPGrabGlue::collectionMovedWrapper(NNTPGrabGlue *glue, const char *collection_name, int old_position, int new_position, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->collectionMoved(QString(collection_name), old_position, new_position);
}

void QNNTPGrabGlue::collectionDownloadedWrapper(NNTPGrabGlue *glue, const char *collection_name, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->collectionDownloaded(QString(collection_name));
}

void QNNTPGrabGlue::pluginLoadedWrapper(NNTPGrabGlue *glue, const char *plugin_name, bool is_persistent, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->pluginLoaded(QString(plugin_name), is_persistent);
}

void QNNTPGrabGlue::pluginUnloadedWrapper(NNTPGrabGlue *glue, const char *plugin_name, void *data)
{
    Q_UNUSED(glue);

    emit ((QNNTPGrabGlue*) data)->pluginUnloaded(QString(plugin_name));
}

void QNNTPGrabGlue::pluginEventWrapper(NNTPGrabGlue *glue, const char *plugin_name, const char *event_name, const char **values, void *data)
{
    Q_UNUSED(glue);

    int i;
    QList<QString> list_values;

    i = 0;
    while (values[i]) {
        list_values.append(QString(values[i]));
        i++;
    }

    emit ((QNNTPGrabGlue*) data)->pluginEvent(QString(plugin_name), QString(event_name), list_values);
}

/*************************
 * Public methods        *
 *************************/
bool QNNTPGrabGlue::init(QString hostname, int port, QString username, QString password, bool use_ssl, QString &errmsg, QString &warnings)
{
    char *errmsg_tmp = NULL;
    char *warnings_tmp = NULL;

    nntpgrab_utils_perform_base_initialization();

    glue = nntpgrab_glue_new();

    if (!nntpgrab_glue_init(glue, NNTPGRAB_GLUE_VERSION, &errmsg_tmp)) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);

        nntpgrab_glue_destroy(glue);
        glue = NULL;
        return false;
    }

    const char *hostname_s = hostname.toUtf8();
    const char *username_s = username.toUtf8();
    const char *password_s = password.toUtf8();

    if (hostname_s && strlen(hostname_s) == 0) {
        hostname_s = NULL;
    }
    if (username_s && strlen(username_s) == 0) {
        username_s = NULL;
    }
    if (password_s && strlen(password_s) == 0) {
        password_s = NULL;
    }

    if (!nntpgrab_glue_connect(glue, hostname_s, port, username_s, password_s, use_ssl, &errmsg_tmp, &warnings_tmp)) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);

        if (warnings_tmp) {
            warnings.clear();
            warnings.append(warnings_tmp);
            ng_free(warnings_tmp);
        }

        return false;
    }

    if (warnings_tmp) {
        warnings.clear();
        warnings.append(warnings_tmp);
        ng_free(warnings_tmp);
    }

    ng_signal_connect(glue, "config_changed", NG_CALLBACK(configChangedWrapper), this);
    ng_signal_connect(glue, "all_downloads_completed", NG_CALLBACK(allDownloadsCompletedWrapper), this);
    ng_signal_connect(glue, "part_download_start", NG_CALLBACK(partDownloadStartWrapper), this);
    ng_signal_connect(glue, "part_done", NG_CALLBACK(partDoneWrapper), this);
    ng_signal_connect(glue, "part_failed", NG_CALLBACK(partFailedWrapper), this);
    ng_signal_connect(glue, "traffic_monitor_update", NG_CALLBACK(trafficMonitorUpdateWrapper), this);
    ng_signal_connect(glue, "part_progress_update", NG_CALLBACK(partProgressUpdateWrapper), this);
    ng_signal_connect(glue, "collection_added", NG_CALLBACK(collectionAddedWrapper), this);
    ng_signal_connect(glue, "collection_modified", NG_CALLBACK(collectionModifiedWrapper), this);
    ng_signal_connect(glue, "collection_removed", NG_CALLBACK(collectionRemovedWrapper), this);
    ng_signal_connect(glue, "file_added", NG_CALLBACK(fileAddedWrapper), this);
    ng_signal_connect(glue, "file_removed", NG_CALLBACK(fileRemovedWrapper), this);
    ng_signal_connect(glue, "file_download_state_update", NG_CALLBACK(fileDownloadStateUpdateWrapper), this);
    ng_signal_connect(glue, "file_state_changed", NG_CALLBACK(fileStateChangedWrapper), this);
    ng_signal_connect(glue, "connection_connecting", NG_CALLBACK(connectionConnectingWrapper), this);
    ng_signal_connect(glue, "connection_connected", NG_CALLBACK(connectionConnectedWrapper), this);
    ng_signal_connect(glue, "connection_disconnect", NG_CALLBACK(connectionDisconnectWrapper), this);
    ng_signal_connect(glue, "schedular_state_changed", NG_CALLBACK(schedularStateChangedWrapper), this);
    ng_signal_connect(glue, "log_message", NG_CALLBACK(logMessageWrapper), this);
    ng_signal_connect(glue, "task_moved", NG_CALLBACK(fileMovedWrapper), this);
    ng_signal_connect(glue, "collection_moved", NG_CALLBACK(collectionMovedWrapper), this);
    ng_signal_connect(glue, "collection_downloaded", NG_CALLBACK(collectionDownloadedWrapper), this);
    ng_signal_connect(glue, "plugin_loaded", NG_CALLBACK(pluginLoadedWrapper), this);
    ng_signal_connect(glue, "plugin_unloaded", NG_CALLBACK(pluginUnloadedWrapper), this);
    ng_signal_connect(glue, "plugin_event", NG_CALLBACK(pluginEventWrapper), this);

    /* If Qt is compiled with GLib support then there's already an GLib main loop active */
    /* If this isn't the case, start one in a seperate thread */
    if (ng_event_handler_loop_depth() == 0) {
        this->callbacks_thread = new QNNTPGrabGlueThread(glue);
        this->callbacks_thread->start();
    }

    return true;
}

bool QNNTPGrabGlue::getIsConnected()
{
    return nntpgrab_glue_get_is_connected(glue);
}

bool QNNTPGrabGlue::getIsStandalone()
{
    return nntpgrab_glue_get_is_standalone(glue);
}

QList<QString> QNNTPGrabGlue::configGetAvailServers()
{
    QList<QString> ret;
    NGList *servers, *list;

    servers = nntpgrab_config_get_avail_servers(glue);
    list = servers;
    while (list) {
        const char *servername = (const char *) list->data;

        ret.append(QString(servername));

        list = ng_list_next(list);
    }
    nntpgrab_config_free_avail_servers(glue, servers);

    return ret;
}

bool QNNTPGrabGlue::configGetServerInfo(QString servername, ConfigServer *server)
{
    return nntpgrab_config_get_server_info(glue, servername.toUtf8(), server);
}

bool QNNTPGrabGlue::configAddServer(ConfigServer new_server, QString &errmsg)
{
    char *errmsg_tmp = NULL;
    bool ret = nntpgrab_config_add_server(glue, new_server, &errmsg_tmp);

    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }

    return ret;
}

bool QNNTPGrabGlue::configDelServer(QString servername, QString &errmsg)
{
    char *errmsg_tmp = NULL;
    bool ret = nntpgrab_config_del_server(glue, servername.toUtf8(), &errmsg_tmp);

    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }

    return ret;
}

bool QNNTPGrabGlue::configEditServer(QString servername, ConfigServer server, QString &errmsg)
{
    char *errmsg_tmp = NULL;
    bool ret = nntpgrab_config_edit_server(glue, servername.toUtf8(), server, &errmsg_tmp);

    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }

    return ret;
}

ConfigOpts QNNTPGrabGlue::configGetOpts()
{
    return nntpgrab_config_get_opts(glue);
}

void QNNTPGrabGlue::configSetOpts(ConfigOpts opts)
{
    nntpgrab_config_set_opts(glue, opts);
}

bool QNNTPGrabGlue::configGetFolderListing(QString parent, QList<QNNTPGrabGlue::Folder> folders)
{
    NGList *list_folders, *list;

    if (!nntpgrab_config_get_folder_listing(glue, parent.toUtf8(), &list_folders)) {
        return false;
    }

    list = list_folders;
    while (list) {
        NNTPGrabFolder *item = (NNTPGrabFolder*) list->data;
        QNNTPGrabGlue::Folder new_item;

        new_item.folder = item->folder;
        new_item.has_subfolders = item->has_subfolders;

        folders.append(new_item);

        list = ng_list_next(list);
    }
    nntpgrab_config_free_folder_listing(glue, list_folders);

    return true;
}

bool QNNTPGrabGlue::startSchedular()
{
    return nntpgrab_schedular_start(this->glue);
}

bool QNNTPGrabGlue::stopSchedular(bool wait)
{
    return nntpgrab_schedular_stop(this->glue, wait);
}

QNNTPGrabGlue::SchedularState QNNTPGrabGlue::getSchedularState()
{
    return (QNNTPGrabGlue::SchedularState) nntpgrab_schedular_get_state(this->glue);
}

bool QNNTPGrabGlue::schedularAddTaskToQueue(QString collection_name, QString subject, QString poster, QDateTime stamp, quint64 file_size, QList<QString> groups, QList<QNNTPGrabGlue::Part> parts, QString &errmsg)
{
    NGList *list_groups = NULL;
    NGList *list_parts = NULL;
    NGList *list;
    bool ret;
    int i;
    char *errmsg_tmp = NULL;

    for (i = 0; i < groups.length(); i++) {
        list_groups = ng_list_append(list_groups, (char*) strdup(groups[i].toUtf8().data()));
    }
    for (i = 0; i < parts.length(); i++) {
        QNNTPGrabGlue::Part part = parts.at(i);
        NNTPGrabPart *part_tmp = (NNTPGrabPart*) calloc(sizeof(NNTPGrabPart), 1);

        strncpy(part_tmp->message_id, part.message_id.toUtf8().data(), sizeof(part_tmp->message_id) - 1);
        part_tmp->part_num = part.part_num;
        part_tmp->size = part.size;

        list_parts = ng_list_append(list_parts, part_tmp);
    }
    ret = nntpgrab_schedular_add_task_to_queue(glue, collection_name.toUtf8(), subject.toUtf8(), poster.toUtf8(), stamp.toTime_t(), file_size, list_groups, list_parts, &errmsg_tmp);

    list = list_groups;
    while (list) {
        free(list->data);
        list = ng_list_next(list);
    }

    list = list_parts;
    while (list) {
        free(list->data);
        list = ng_list_next(list);
    }

    ng_list_free(list_groups);
    ng_list_free(list_parts);

    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }
    return ret;
}

bool QNNTPGrabGlue::schedularDelTaskFromQueue(QString collection_name, QString subject, QString &errmsg)
{
    char *errmsg_tmp = NULL;
    bool ret = nntpgrab_schedular_del_task_from_queue(glue, collection_name.toUtf8(), (subject.isEmpty() ? NULL : subject.toUtf8().data()), &errmsg_tmp);

    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }

    return ret;
}

bool QNNTPGrabGlue::schedularRestartTask(QString collection_name, QString subject, QString &errmsg)
{
    char *errmsg_tmp = NULL;
    bool ret = nntpgrab_schedular_restart_task(glue, collection_name.toUtf8(), (subject.isEmpty() ? NULL : subject.toUtf8().data()), &errmsg_tmp);

    if (!ret) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
    }

    return ret;
}

bool QNNTPGrabGlue::schedularMarkTaskOptional(QString collection_name, QString subject, bool is_optional)
{
    return nntpgrab_schedular_mark_task_optional(glue, collection_name.toUtf8(), subject.toUtf8(), is_optional);
}

bool QNNTPGrabGlue::schedularSaveQueue(char **errmsg)
{
    return nntpgrab_schedular_save_queue(glue, errmsg);
}

struct _foreach_cb_data {
    QList<QNNTPGrabGlue::Collection> collections;
    QNNTPGrabGlue::Collection *active_collection;
    QNNTPGrabGlue::File *active_file;
};

static void foreach_collection_func(const char *collection_name, const char *poster, nguint64 total_size, nguint64 total_size_remaining, int position, void *data)
{
    struct _foreach_cb_data *cb_data = (struct _foreach_cb_data*) data;
    QNNTPGrabGlue::Collection collection;

    collection.collection_name = QString(collection_name);
    collection.poster = QString(poster);
    collection.total_size = total_size;
    collection.total_size_remaining = total_size_remaining;
    collection.position = position;

    cb_data->collections.append(collection);
    cb_data->active_collection = &cb_data->collections[cb_data->collections.count() - 1];
    cb_data->active_file = NULL;
}

static void foreach_file_func(const char *collection_name, const char *subject, const char *poster, ngint64 stamp, nguint64 file_size, nguint64 file_size_remaining, int position, int num_parts_total, int num_parts_downloaded, int num_parts_failed, NGTaskState status, const char *filename, void *data)
{
    struct _foreach_cb_data *cb_data = (struct _foreach_cb_data*) data;
    QNNTPGrabGlue::File file;

    file.collection_name = QString(collection_name);
    file.subject = QString(subject);
    file.poster = QString(poster);
    file.stamp = QDateTime::fromTime_t(stamp);
    file.file_size = file_size;
    file.file_size_remaining = file_size_remaining;
    file.position = position;
    file.num_parts_total = num_parts_total;
    file.num_parts_downloaded = num_parts_downloaded;
    file.num_parts_failed = num_parts_failed;
    file.status = (QNNTPGrabGlue::TaskState) status;
    file.filename = QString(filename);

    Q_ASSERT(cb_data->active_collection != NULL);
    Q_ASSERT(cb_data->active_collection->collection_name == QString(collection_name));

    cb_data->active_collection->files.append(file);
    cb_data->active_file = &cb_data->active_collection->files[cb_data->active_collection->files.count() - 1];
}

static void foreach_group_func (const char *collection_name, const char *subject, const char *group, void *data)
{
    struct _foreach_cb_data *cb_data = (struct _foreach_cb_data*) data;

    Q_ASSERT(cb_data->active_file != NULL);
    Q_ASSERT(cb_data->active_file->collection_name == QString(collection_name));
    Q_ASSERT(cb_data->active_file->subject == QString(subject));

    cb_data->active_file->groups.append(QString(group));
}

QList<QNNTPGrabGlue::Collection> QNNTPGrabGlue::schedularGetAllTasks(void)
{
    struct _foreach_cb_data cb_data;

    nntpgrab_schedular_foreach_task(glue, foreach_collection_func, foreach_file_func, foreach_group_func, &cb_data);

    return cb_data.collections;
}

bool QNNTPGrabGlue::schedularMoveTask(QString collection_name_src, QString subject_src, QString collection_name_dest, int position_dest)
{
    return nntpgrab_schedular_move_task(glue, collection_name_src.toUtf8(), subject_src.toUtf8(), collection_name_dest.toUtf8(), position_dest);
}

bool QNNTPGrabGlue::schedularMoveCollection(QString collection_name, int new_position)
{
    return nntpgrab_schedular_move_collection(glue, collection_name.toUtf8(), new_position);
}

QList<QString> QNNTPGrabGlue::getAvailPlugins()
{
    QList<QString> ret;
    NGList *plugins, *list;

    plugins = nntpgrab_plugins_get_avail_plugins(glue);
    list = plugins;
    while (list) {
        const char *plugin_name = (const char *) list->data;

        ret.append(QString(plugin_name));

        list = ng_list_next(list);
    }
    nntpgrab_plugins_free_avail_plugins(glue, plugins);

    return ret;
}

bool QNNTPGrabGlue::getPluginInfo(QString plugin_name, NNTPGrabPluginInfo *plugin_info)
{
    return nntpgrab_plugins_get_plugin_info(glue, plugin_name.toUtf8(), plugin_info);
}

bool QNNTPGrabGlue::loadPlugin(QString plugin_name, QString *errmsg)
{
    bool ret;
    char *errmsg_tmp = NULL;

    ret = nntpgrab_plugins_load_plugin(this->glue, plugin_name.toAscii(), &errmsg_tmp);
    if (!ret) {
        errmsg->clear();
        errmsg->append(errmsg_tmp);
    }

    return ret;
}

bool QNNTPGrabGlue::unloadPlugin(QString plugin_name, char **errmsg)
{
    return nntpgrab_plugins_unload_plugin(glue, plugin_name.toUtf8(), errmsg);
}

bool QNNTPGrabGlue::setPluginPersistent(QString plugin_name, bool persistent)
{
    return nntpgrab_plugins_set_persistent(glue, plugin_name.toUtf8(), persistent);
}

QString QNNTPGrabGlue::utilsCalculateFileSize(quint64 file_size)
{
    char buf[16];
    memset(&buf, 0, sizeof(buf));

    nntpgrab_utils_calculate_file_size(file_size, (char*) &buf, sizeof(buf));

    return QString(buf);
}

int QNNTPGrabGlue::utilsCalculateEstimatedTimeRemaining(int bytes_received1, int bytes_received2, int bytes_received3, int bytes_received4, int bytes_received5, int bytes_received6, int bytes_received7, int bytes_received8, int bytes_received9, int bytes_received10, quint64 file_size)
{
    return nntpgrab_utils_calculate_estimated_time_remaining(bytes_received1, bytes_received2, bytes_received3, bytes_received4,
                                                             bytes_received5, bytes_received6, bytes_received7, bytes_received8,
                                                             bytes_received9, bytes_received10, file_size);
}

QString QNNTPGrabGlue::utilsGetReadableTimeRemaining(int estimated_time_remaining)
{
    char buf[64];
    memset(&buf, 0, sizeof(buf));

    nntpgrab_utils_get_readable_time_remaining(estimated_time_remaining, (char*) &buf, sizeof(buf));

    return QString(buf);
}

QString QNNTPGrabGlue::utilsGetReadableFinishedTime(int estimated_time_remaining)
{
    char buf[64];
    memset(&buf, 0, sizeof(buf));

    nntpgrab_utils_get_readable_finished_time(estimated_time_remaining, (char*) &buf, sizeof(buf));

    return QString(buf);
}

bool QNNTPGrabGlue::utilsParseNZBFile(QString nzb_data, QNNTPGrabGlue::NZB &nzbfile, QString &errmsg)
{
    char *errmsg_tmp = NULL;
    NNTPGrabNZB *nzbfile_tmp = nntpgrab_utils_parse_nzb_file(nzb_data.toUtf8().data(), &errmsg_tmp);
    NGList *list;

    if (!nzbfile_tmp) {
        errmsg.clear();
        errmsg.append(errmsg_tmp);
        ng_free(errmsg_tmp);
        return FALSE;
    }

    list = nzbfile_tmp->files;
    while (list) {
        NNTPGrabNZBFile *file_tmp = (NNTPGrabNZBFile*) list->data;

        QNNTPGrabGlue::NZBFile file;

        file.subject = file_tmp->subject;
        file.poster = file_tmp->poster;
        file.stamp = QDateTime::fromTime_t(file_tmp->stamp);
        file.file_size = file_tmp->filesize;

        NGList *list2;

        list2 = file_tmp->segments;
        while (list2) {
            NNTPGrabPart *part_tmp = (NNTPGrabPart*) list2->data;

            QNNTPGrabGlue::Part part;

            part.message_id = part_tmp->message_id;
            part.part_num = part_tmp->part_num;
            part.size = part_tmp->size;

            file.segments.append(part);

            list2 = ng_list_next(list2);
        }

        list2 = file_tmp->groups;
        while (list2) {
            const char *groupname = (const char*) list2->data;

            file.groups.append(QString(groupname));

            list2 = ng_list_next(list2);
        }

        nzbfile.files.append(file);

        list = ng_list_next(list);
    }

    nntpgrab_utils_nzb_file_free(nzbfile_tmp);

    return TRUE;
}
