#ifndef QNNTPGRAB_GLUE_H
#define QNNTPGRAB_GLUE_H

#include <QObject>
#include <QString>
#include <QThread>
#include <QDateTime>

#include "nntpgrab_glue.h"

class QNNTPGrabGlueThread : public QThread
{
    Q_OBJECT

private:
    NNTPGrabGlue *glue;

public:
    QNNTPGrabGlueThread (NNTPGrabGlue *glue);
    ~QNNTPGrabGlueThread();

protected:
    void run();
};

class QNNTPGrabGlue : public QObject
{
    Q_OBJECT

public:
    QNNTPGrabGlue();
    ~QNNTPGrabGlue();

    enum NNTPDisconnectType {
        DISCONNECT_NORMAL,
        DISCONNECT_NO_SUCH_HOST,
        DISCONNECT_CONNECTION_REFUSED,
        DISCONNECT_TOO_MANY_CONNECTIONS,
        DISCONNECT_CONNECT_TIMEOUT,
        DISCONNECT_READ_ERROR,
        DISCONNECT_READ_TIMEOUT,
        DISCONNECT_WRITE_ERROR,
        DISCONNECT_IDLE_TIMEOUT,
        DISCONNECT_INVALID_MSG,
        DISCONNECT_LOGIN_FAILURE,
        DISCONNECT_ERROR_SSL_INITIALISE,
        DISCONNECT_UNEXPECTED
    };

    enum TaskState {
        TASK_STATE_WAITING_FOR_DOWNLOAD,
        TASK_STATE_DOWNLOADING,
        TASK_STATE_WAITING_FOR_DECODE,
        TASK_STATE_DECODING,
        TASK_STATE_FINISHED_COMPLETE,
        TASK_STATE_FINISHED_INCOMPLETE,
        TASK_STATE_FINISHED_NO_PARTS_AVAIL,
        TASK_STATE_SKIPPED
    };

    enum LogLevel {
        LOG_LEVEL_ALL = 0,
        LOG_LEVEL_INFO,
        LOG_LEVEL_WARNING,
        LOG_LEVEL_ERROR,
        LOG_LEVEL_FATAL,
        LOG_LEVEL_DEBUG,
    };

    enum SchedularState {
        SCHEDULAR_STATE_RUNNING,
        SCHEDULAR_STATE_STOPPING,
        SCHEDULAR_STATE_STOPPED
    };

    struct Folder {
        QString folder;
        bool has_subfolders;
    };

    struct File {
        QString collection_name;
        QString subject;
        QString poster;
        QDateTime stamp;
        quint64 file_size;
        quint64 file_size_remaining;
        int position;
        int num_parts_total;
        int num_parts_downloaded;
        int num_parts_failed;
        QNNTPGrabGlue::TaskState status;
        QString filename;
        QList<QString> groups;
    };

    struct Collection {
        QString collection_name;
        QString poster;
        quint64 total_size;
        quint64 total_size_remaining;
        int position;
        QList<QNNTPGrabGlue::File> files;
    };

    struct Part {
        QString message_id;
        int size;
        int part_num;
    };

    struct NZBFile {
        QString subject;
        QString poster;
        QDateTime stamp;
        quint64 file_size;
        QList<QString> groups;
        QList<QNNTPGrabGlue::Part> segments;
    };

    struct NZB {
        QList<QNNTPGrabGlue::NZBFile> files;
    };

    bool init(QString hostname, int port, QString username, QString password, bool use_ssl, QString &errmsg, QString &warnings);
    bool getIsConnected();
    bool getIsStandalone();

    QList<QString> configGetAvailServers();
    bool configGetServerInfo(QString servername, ConfigServer *server);
    bool configAddServer(ConfigServer new_server, QString &errmsg);
    bool configDelServer(QString servername, QString &errmsg);
    bool configEditServer(QString servername, ConfigServer server, QString &errmsg);

    ConfigOpts configGetOpts();
    void configSetOpts(ConfigOpts opts);

    bool configGetFolderListing(QString parent, QList<QNNTPGrabGlue::Folder> folders);

    bool startSchedular();
    bool stopSchedular(bool wait);
    SchedularState getSchedularState();

    bool schedularAddTaskToQueue(QString collection_name, QString subject, QString poster, QDateTime stamp, quint64 file_size, QList<QString> groups, QList<QNNTPGrabGlue::Part> parts, QString &errmsg);
    bool schedularDelTaskFromQueue(QString collection_name, QString subject, QString &errmsg);
    bool schedularRestartTask(QString collection_name, QString subject, QString &errmsg);
    bool schedularMarkTaskOptional(QString collection_name, QString subject, bool is_optional);
    bool schedularSaveQueue(char **errmsg);

    QList<QNNTPGrabGlue::Collection> schedularGetAllTasks(void);

    bool schedularMoveTask(QString collection_name_src, QString subject_src, QString collection_name_dest, int position_dest);
    bool schedularMoveCollection(QString collection_name, int new_position);

    QList<QString> getAvailPlugins();
    bool getPluginInfo(QString plugin_name, NNTPGrabPluginInfo *plugin_info);
    bool loadPlugin(QString plugin_name, QString *errmsg);
    bool unloadPlugin(QString plugin_name, char **errmsg);
    bool setPluginPersistent(QString plugin_name, bool persistent);

    QString utilsCalculateFileSize(quint64 file_size);
    int utilsCalculateEstimatedTimeRemaining(int bytes_received1, int bytes_received2, int bytes_received3, int bytes_received4, int bytes_received5, int bytes_received6, int bytes_received7, int bytes_received8, int bytes_received9, int bytes_received10, quint64 file_size);
    QString utilsGetReadableTimeRemaining(int estimated_time_remaining);
    QString utilsGetReadableFinishedTime(int estimated_time_remaining);

    bool utilsParseNZBFile(QString nzb_data, QNNTPGrabGlue::NZB &nzbfile, QString &errmsg);
private:
    NNTPGrabGlue *glue;
    QNNTPGrabGlueThread *callbacks_thread;

    static void configChangedWrapper(NNTPGrabGlue *glue, void *data);
    static void allDownloadsCompletedWrapper(NNTPGrabGlue *glue, void *data);
    static void partDownloadStartWrapper(NNTPGrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, void *data);
    static void partDoneWrapper(NNTPGrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, int size, void *data);
    static void partFailedWrapper(NNTPGrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num,int size, bool all_servers_tried, void *data);
    static void trafficMonitorUpdateWrapper(NNTPGrabGlue *glue, int bytes_received1, int bytes_received2, int bytes_received3, int bytes_received4, int bytes_received5, int bytes_received6, int bytes_received7, int bytes_received8, int bytes_received9, int bytes_received10, nguint64 stamp, double average, void *data);
    static void partProgressUpdateWrapper(NNTPGrabGlue *glue, const char *servername, int conn_id, const char *collection_name, const char *subject, int part_num, int bytes_downloaded, int bytes_total, void *data);
    static void collectionAddedWrapper(NNTPGrabGlue *glue, const char *collection_name, const char *poster, void *data);
    static void collectionModifiedWrapper(NNTPGrabGlue *glue, const char *collection_name, const char *poster, void *data);
    static void collectionRemovedWrapper(NNTPGrabGlue *glue, const char *collection_name, void *data);
    static void fileAddedWrapper(NNTPGrabGlue *glue, const char *collection_name, const char *subject, const char *poster, nguint64 stamp, nguint64 file_size, nguint64 total_size, nguint64 total_size_remaining, TaskState state, int num_parts, NGList *groups, void *data);
    static void fileRemovedWrapper(NNTPGrabGlue *glue, const char *collection_name, const char *subject, nguint64 total_size, nguint64 total_size_remaining, void *data);
    static void fileDownloadStateUpdateWrapper(NNTPGrabGlue *glue, const char *collection_name, const char *subject, int num_parts_total, int num_parts_done, int num_parts_failed, nguint64 file_size, nguint64 file_size_remaining, nguint64 total_size, nguint64 total_size_remaining, void *data);
    static void fileStateChangedWrapper(NNTPGrabGlue *glue, const char *collection_name, const char *subject, const char *real_filename, TaskState old_state, TaskState new_state, void *data);
    static void connectionConnectingWrapper(NNTPGrabGlue *glue, const char *servername, int conn_id, void *data);
    static void connectionConnectedWrapper(NNTPGrabGlue *glue, const char *servername, int conn_id, const char *welcome_msg, void *data);
    static void connectionDisconnectWrapper(NNTPGrabGlue *glue, const char *servername, int conn_id, NNTPDisconnectType disconnect_type, const char *reason, void *data);
    static void schedularStateChangedWrapper(NNTPGrabGlue *glue, NGSchedularState new_state, const char *reason, void *data);
    static void logMessageWrapper(NNTPGrabGlue *glue, const char *component, NGLogLevel log_level, const char *msg, void *data);
    static void fileMovedWrapper(NNTPGrabGlue *glue, const char *orig_collection_name, const char *subject, const char *new_collection_name, int old_position, int new_position, void *data);
    static void collectionMovedWrapper(NNTPGrabGlue *glue, const char *collection_name, int old_position, int new_position, void *data);
    static void collectionDownloadedWrapper(NNTPGrabGlue *glue, const char *collection_name, void *data);
    static void pluginLoadedWrapper(NNTPGrabGlue *glue, const char *plugin_name, bool is_persistent, void *data);
    static void pluginUnloadedWrapper(NNTPGrabGlue *glue, const char *plugin_name, void *data);
    static void pluginEventWrapper(NNTPGrabGlue *glue, const char *plugin_name, const char *event_name, const char **values, void *data);

signals:
    void configChanged();
    void allDownloadsCompleted();
    void partDownloadStart(QString servername, int conn_id, QString collection_name, QString subject, int part_num);
    void partDone(QString servername, int conn_id, QString collection_name, QString subject, int part_num, int size);
    void partFailed(QString servername, int conn_id, QString collection_name, QString subject, int part_num, int size, bool all_servers_tried);
    void partProgressUpdate(QString servername, int conn_id, QString collection_name, QString subject, int part_num, int bytes_downloaded, int bytes_total);
    void trafficMonitorUpdate(int bytes_received1, int bytes_received2, int bytes_received3, int bytes_received4, int bytes_received5, int bytes_received6, int bytes_received7, int bytes_received8, int bytes_received9, int bytes_received10, time_t stamp, double average);
    void collectionAdded(QString collection_name, QString poster);
    void collectionModified(QString collection_name, QString poster);
    void collectionRemoved(QString collection_name);
    void fileAdded(QString collection_name, QString subject, QString poster, time_t stamp, quint64 file_size, quint64 total_size, quint64 total_size_remaining, QNNTPGrabGlue::TaskState state, int num_parts, QList<QString> groups);
    void fileRemoved(QString collection_name, QString subject, quint64 total_size, quint64 total_size_remaining);
    void fileDownloadStateUpdate(QString collection_name, QString subject, int num_parts_total, int num_parts_done, int num_parts_failed, quint64 file_size, quint64 file_size_remaining, quint64 total_size, quint64 total_size_remaining);
    void fileStateChanged(QString collection_name, QString subject, QString real_filename, QNNTPGrabGlue::TaskState old_state, QNNTPGrabGlue::TaskState new_state);
    void connectionConnecting(QString servername, int conn_id);
    void connectionConnected(QString servername, int conn_id, QString welcome_msg);
    void connectionDisconnect(QString servername, int conn_id, QNNTPGrabGlue::NNTPDisconnectType disconnect_type, QString reason);
    void schedularStateChanged(QNNTPGrabGlue::SchedularState new_state, QString reason);
    void logMessage(QString component, QNNTPGrabGlue::LogLevel log_level, QString msg);
    void fileMoved(QString orig_collection_name, QString subject, QString new_collection_name, int old_position, int new_position);
    void collectionMoved(QString collection_name, int old_position, int new_position);
    void collectionDownloaded(QString collection_name);
    void pluginLoaded(QString plugin_name, bool is_persistent);
    void pluginUnloaded(QString plugin_name);
    void pluginEvent(QString plugin_name, QString event_name, QList<QString> values);
};

#endif // QNNTPGRAB_GLUE_H
