#ifndef WIDGETDOWNLOADQUEUE_H
#define WIDGETDOWNLOADQUEUE_H

#include <QWidget>
#include <QNNTPGrabGlue.h>
#include <QTreeWidgetItem>
#include "ProgressBarDelegateDownloadQueue.h"

namespace Ui {
    class WidgetDownloadQueue;
}

class WidgetDownloadQueue : public QWidget {
    Q_OBJECT
public:
    WidgetDownloadQueue(QNNTPGrabGlue *glue, QAction *actionRemoveSelectedFiles, QWidget *parent = 0);
    ~WidgetDownloadQueue();

protected:
    void changeEvent(QEvent *e);

private:
    Ui::WidgetDownloadQueue *ui;
    QNNTPGrabGlue *glue;
    ProgressBarDelegateDownloadQueue *progressbarDelegate;
    QAction *actionRemoveSelectedFiles;

    enum moveDirection {
        MOVE_DIRECTION_TOP,
        MOVE_DIRECTION_UP,
        MOVE_DIRECTION_DOWN,
        MOVE_DIRECTION_BOTTOM
    };

    void doMoveItem(QString collection_name, QString subject, int new_position);
    int calculateNewPosition(int current_position, moveDirection direction);
    void processSelectionAndMoveItems(moveDirection direction);

    int calculateProgress(quint64 total_file_size, quint64 file_size_remaining);
    QTreeWidgetItem *getTreeWidgetByCollection(QString collection_name);
    QTreeWidgetItem *getTreeWidgetBySubject(QTreeWidgetItem *collection_node, QString subject);

private slots:
    void btnQueueMoveToTop_clicked(void);
    void btnQueueMoveUp_clicked(void);
    void btnQueueMoveDown_clicked(void);
    void btnQueueMoveToBottom_clicked(void);

    void onFileDownloadStateUpdate(QString collection_name, QString subject, int num_parts_total, int num_parts_done, int num_parts_failed, quint64 file_size, quint64 file_size_remaining, quint64 total_size, quint64 total_size_remaining);
    void onFileStateChanged(QString collection_name, QString subject, QString real_filename, QNNTPGrabGlue::TaskState old_state, QNNTPGrabGlue::TaskState new_state);

    void onFileAdded(QString collection_name, QString subject, QString poster, time_t stamp, quint64 file_size, quint64 total_size, quint64 total_size_remaining, QNNTPGrabGlue::TaskState state, int num_parts, QList<QString> groups);
    void onFileRemoved(QString collection_name, QString subject, quint64 total_size, quint64 total_size_remaining);
    void onFileMoved(QString orig_collection_name, QString subject, QString new_collection_name, int old_position, int new_position);

    void onCollectionAdded(QString collection_name, QString poster);
    void onCollectionRemoved(QString collection_name);
    void onCollectionModified(QString collection_name, QString poster);
    void onCollectionMoved(QString collection_name, int old_position, int new_position);

    void onTrafficMonitorUpdate(int bytes_received1, int bytes_received2, int bytes_received3, int bytes_received4, int bytes_received5, int bytes_received6, int bytes_received7, int bytes_received8, int bytes_received9, int bytes_received10, time_t stamp, double average);

    void onItemSelectionChanged(void);

    void actionRemoveSelectedFiles_activated(void);
    void actionRestartSelectedItems_activated(void);
    void actionMarkItemOptional_activated(void);
    void actionOpenDownloadFolder_activated(void);
    void actionExecuteSelectedItem_activated(void);

    void showContextMenu(const QPoint &position);
};

#endif // WIDGETDOWNLOADQUEUE_H
