#include <QtGui/QApplication>
#include <QTranslator>
#include <QLocale>
#include <QLibraryInfo>
#include "DialogConnect.h"
#include <QMessageBox>
#include <stdio.h>

#ifdef Q_OS_MAC
#include <CoreFoundation/CFBundle.h>
#endif


static void messagehandler(QtMsgType type, const char *msg);

int main(int argc, char *argv[])
{
    qInstallMsgHandler(messagehandler);
    QApplication a(argc, argv);

    QTranslator qtTranslator;
    qtTranslator.load("qt_" + QLocale::system().name(), QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    a.installTranslator(&qtTranslator);

    QTranslator nntpgrabTranslator;
#ifdef Q_OS_LINUX
    /* Yes this is very ugly..but as QMake doesn't have a default path for translations on Linux environments, this is the best we have */
    nntpgrabTranslator.load("nntpgrab_" + QLocale::system().name(), QString("/usr/share/nntpgrab/translations"));
#elif defined(Q_OS_WIN32)
    nntpgrabTranslator.load("nntpgrab_" + QLocale::system().name(), QCoreApplication::applicationDirPath() + QString("/../share/nntpgrab/translations"));
#elif defined(Q_OS_MAC)
    CFURLRef appUrlRef = CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle());
    CFStringRef macPath = CFURLCopyFileSystemPath(appUrlRef, kCFURLPOSIXPathStyle);
    QString resources_path = CFStringGetCStringPtr(macPath, CFStringGetSystemEncoding());
    CFRelease(appUrlRef);
    CFRelease(macPath);

    appUrlRef = CFBundleCopyBundleURL(CFBundleGetMainBundle());
    macPath = CFURLCopyFileSystemPath(appUrlRef, kCFURLPOSIXPathStyle);
    QString bundle_path = CFStringGetCStringPtr(macPath, CFStringGetSystemEncoding());
    CFRelease(appUrlRef);
    CFRelease(macPath);

    nntpgrabTranslator.load("nntpgrab_" + QLocale::system().name(), bundle_path + QString("/") + resources_path + QString("/share/nntpgrab/translations"));
#else
    nntpgrabTranslator.load("nntpgrab_" + QLocale::system().name());
#endif
    a.installTranslator(&nntpgrabTranslator);

    DialogConnect d;
    d.show();
    return a.exec();
}

static void messagehandler(QtMsgType type, const char *msg)
{
    switch (type) {
    case QtDebugMsg:
        puts(msg);
        break;
    case QtWarningMsg:
    case QtCriticalMsg:
        QMessageBox::warning(NULL,
                             "NNTPGrab Warning",
                             msg,
                             QMessageBox::Ok);
        break;        
    case QtFatalMsg:
        QMessageBox::critical(NULL,
                              "NNTPGrab Error",
                              msg,
                              QMessageBox::Ok);
        QCoreApplication::exit(1);
        break;
    }
}

