#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QDesktopServices>
#include <QUrl>

#include "DialogManagePlugins.h"
#include "DialogImportNZB.h"

MainWindow::MainWindow(QNNTPGrabGlue *glue, QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    this->glue = glue;
    this->formDebug = NULL;

    ui->setupUi(this);

    /* Create the individual tabs */
    updateInfoWidget = new WidgetUpdateInfo();
    ui->tabWidget->addTab(updateInfoWidget, tr("Information"));

    configWidget = new WidgetConfig(glue);
    ui->tabWidget->addTab(configWidget, tr("Configuration"));

    downloadQueueWidget = new WidgetDownloadQueue(glue, ui->actionRemoveSelectedFiles);
    ui->tabWidget->addTab(downloadQueueWidget, tr("Download queue"));

    nzbcreatorWidget = new WidgetNZBCreator();
    nzbcreatorWidget->setEnabled(false);
    ui->tabWidget->addTab(nzbcreatorWidget, tr("Online Search"));

    repairAndUnpackWidget = new WidgetRepairAndUnpack();
    repairAndUnpackWidget->setEnabled(false);
    ui->tabWidget->addTab(repairAndUnpackWidget, tr("Repair and Unpack"));

    /* Remove the dummy tab */
    ui->tabWidget->removeTab(0);

    /* Create the traffic monitor and list of active connections */
    trafficMonitorWidget = new WidgetTrafficMonitor(glue);
    ui->splitterBottom->addWidget(trafficMonitorWidget);
    connectionsWidget = new WidgetConnections(glue);
    ui->splitterBottom->addWidget(connectionsWidget);

    /* Fix the position of the splitter */
    /* FIXME: This must be implemented in a more proper way.. */
    QList<int> sizes = ui->splitterBottom->sizes();
    sizes[0] = 150;
    sizes[1] = 65535;
    ui->splitterBottom->setSizes(sizes);

    sizes = ui->splitterMain->sizes();
    sizes[0] = 65535;
    sizes[1] = 150;
    ui->splitterMain->setSizes(sizes);

    connect(ui->actionImportNZBFile, SIGNAL(triggered(void)),
            SLOT(actionImportNZB_activated(void)));
    connect(ui->actionPauseDownloadQueue, SIGNAL(triggered(void)),
            SLOT(actionPauseDownloadQueue_activated()));
    connect(ui->actionDonate, SIGNAL(triggered(void)),
            SLOT(actionDonate_activated(void)));
    connect(ui->actionEnableAutomaticShutdown, SIGNAL(triggered(void)),
            SLOT(actionEnableAutomaticShutdown_activated()));

    connect(ui->actionManagePlugins, SIGNAL(triggered(void)),
            SLOT(actionManagePlugins_activated(void)));
    connect(ui->actionShowDebugWindow, SIGNAL(triggered(void)),
            SLOT(actionShowDebugWindow_activated(void)));

    connect(glue, SIGNAL(schedularStateChanged(QNNTPGrabGlue::SchedularState,QString)), SLOT(onSchedularStateChanged(QNNTPGrabGlue::SchedularState,QString)));

    glue->startSchedular();
}

MainWindow::~MainWindow()
{
    delete updateInfoWidget;
    delete configWidget;
    delete downloadQueueWidget;
    delete nzbcreatorWidget;
    delete repairAndUnpackWidget;

    delete trafficMonitorWidget;
    delete connectionsWidget;

    delete ui;

    if (formDebug) {
        delete formDebug;
    }

    if (glue) {
        delete glue;
    }
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::actionImportNZB_activated(void)
{
    DialogImportNZB d(glue);

    qDebug("import NZB");

    d.exec();
}

void MainWindow::actionDonate_activated(void)
{
    QDesktopServices::openUrl(QUrl("http://www.nntpgrab.nl/donate.php"));
}

void MainWindow::actionEnableAutomaticShutdown_activated(void)
{
    qDebug("enable automatic shutdown");
}

void MainWindow::actionManagePlugins_activated(void)
{
    DialogManagePlugins d(glue);

    qDebug("manage plugins");

    d.exec();
}

void MainWindow::actionShowDebugWindow_activated(void)
{
    if (formDebug) {
        formDebug->show();
        formDebug->activateWindow();
        return;
    }

    formDebug = new FormDebug(glue);
    formDebug->show();
}

void MainWindow::actionPauseDownloadQueue_activated(void)
{
    if (!ui->actionPauseDownloadQueue->isChecked()) {
        /* The user just pressed the button to indicate that the schedular needs to be stopped */
        if (glue->getSchedularState() == QNNTPGrabGlue::SCHEDULAR_STATE_STOPPED) {
            glue->startSchedular();
        } else {
            qWarning(tr("The schedular is currently being stopped").toUtf8().data());
        }
    } else {
        glue->stopSchedular(false);
        ui->actionPauseDownloadQueue->setEnabled(false);
    }
}

void MainWindow::onSchedularStateChanged(QNNTPGrabGlue::SchedularState state, QString reason)
{
    if (state == QNNTPGrabGlue::SCHEDULAR_STATE_RUNNING) {
        ui->actionPauseDownloadQueue->setChecked(false);
        ui->actionPauseDownloadQueue->setEnabled(true);
        ui->actionPauseDownloadQueue->setText(tr("Pause download queue"));
    } else if (state == QNNTPGrabGlue::SCHEDULAR_STATE_STOPPED) {
        ui->actionPauseDownloadQueue->setChecked(true);
        ui->actionPauseDownloadQueue->setEnabled(true);
        ui->actionPauseDownloadQueue->setText(tr("Resume download queue"));
    }

    if (state == QNNTPGrabGlue::SCHEDULAR_STATE_STOPPING && !reason.isEmpty()) {
        qWarning(tr("The schedular has been paused due to an error:\n%s").toUtf8().data(), reason.toUtf8().data());
        ui->actionPauseDownloadQueue->setEnabled(false);
    }
}
