/*
    Copyright (C) 2005-2010  Erik van Pienbroek

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef _NNTPGRAB_GLUE_INTERNAL_H_
#define _NNTPGRAB_GLUE_INTERNAL_H_

#include <glib.h>
#include <glib-object.h>
#include "nntpgrab_glue.h"
#include "nntpgrab.h"

#ifdef  __cplusplus
extern "C" {
#endif

#define READ_TIMEOUT_VALUE      30

enum
{
    CONFIG_CHANGED_SIGNAL,
    PART_DOWNLOAD_START_SIGNAL,
    PART_DONE_SIGNAL,
    PART_FAILED_SIGNAL,
    PART_PROGRESS_UPDATE_SIGNAL,
    TRAFFIC_MONITOR_UPDATE_SIGNAL,
    COLLECTION_ADDED_SIGNAL,
    COLLECTION_REMOVED_SIGNAL,
    COLLECTION_MODIFIED_SIGNAL,
    FILE_ADDED_SIGNAL,
    FILE_REMOVED_SIGNAL,
    FILE_DOWNLOAD_STATE_UPDATE_SIGNAL,
    FILE_STATE_CHANGED_SIGNAL,
    CONNECTION_CONNECTING_SIGNAL,
    CONNECTION_CONNECTED_SIGNAL,
    CONNECTION_DISCONNECT_SIGNAL,
    SCHEDULAR_STATE_CHANGED_SIGNAL,
    LOG_MESSAGE_SIGNAL,
    CONNECTION_LOST_SIGNAL,
    TASK_MOVED_SIGNAL,
    COLLECTION_MOVED_SIGNAL,
    COLLECTION_DOWNLOADED_SIGNAL,
    ALL_DOWNLOADS_COMPLETED_SIGNAL,
    PLUGIN_LOADED_SIGNAL,
    PLUGIN_UNLOADED_SIGNAL,
    PLUGIN_EVENT_SIGNAL,
    LAST_SIGNAL
};

typedef struct _connection {
    int socket_id;
    GString *recv_buf;
} Connection;

struct NNTPGrabGlue
{
    GObject parent;

    Connection socket;

    int glue_version;

    gboolean is_initialized;

    NNTPGrabCore *standalone_core;
};

gboolean write_line(Connection *sock, const char *format, ...);
gboolean has_signal_handler_pending(int signal_id);

void     connect_signal_handlers(NNTPGrabCore *core);
void     disconnect_signal_handlers(NNTPGrabCore *core);

void     glue_process_jsonrpc_msg(const char *msg);

NGList          *nntpgrab_glue_config_get_avail_servers(NNTPGrabGlue *obj);
gboolean         nntpgrab_glue_config_get_server_info(NNTPGrabGlue *obj, const char *servername, ConfigServer *ret);
gboolean         nntpgrab_glue_config_add_server(NNTPGrabGlue *obj, ConfigServer new_server, char **errmsg);
gboolean         nntpgrab_glue_config_del_server(NNTPGrabGlue *obj, const char *servername, char **errmsg);
gboolean         nntpgrab_glue_config_edit_server(NNTPGrabGlue *obj, const char *servername, ConfigServer new_server, char **errmsg);
gboolean         nntpgrab_glue_config_get_opts(NNTPGrabGlue *obj, ConfigOpts *opts);
gboolean         nntpgrab_glue_config_set_opts(NNTPGrabGlue *obj, ConfigOpts opts);
gboolean         nntpgrab_glue_config_get_folder_listing(NNTPGrabGlue *obj, const char *parent, NGList **folders);
gboolean         nntpgrab_glue_schedular_start(NNTPGrabGlue *obj);
gboolean         nntpgrab_glue_schedular_stop(NNTPGrabGlue *obj, gboolean wait);
NGSchedularState nntpgrab_glue_schedular_get_state(NNTPGrabGlue *obj);
gboolean         nntpgrab_glue_schedular_add_task_to_queue(NNTPGrabGlue *obj, const char *collection_name, const char *subject, const char *poster, time_t stamp, nguint64 file_size, NGList *groups, NGList *parts, char **errmsg);
gboolean         nntpgrab_glue_schedular_del_task_from_queue(NNTPGrabGlue *obj, const char *collection_name, const char *subject, char **errmsg);
gboolean         nntpgrab_glue_schedular_restart_task(NNTPGrabGlue *obj, const char *collection_name, const char *subject, char **errmsg);
gboolean         nntpgrab_glue_schedular_save_queue(NNTPGrabGlue *obj, char **errmsg);
gboolean         nntpgrab_glue_schedular_foreach_task(NNTPGrabGlue *obj, ForeachCollectionFunc collection_func, ForeachFileFunc file_func, ForeachGroupFunc group_func, void *data);
gboolean         nntpgrab_glue_schedular_move_task(NNTPGrabGlue *obj, const char *collection_name_src, const char *subject_src, const char *collection_name_dest, int position_dest);
gboolean         nntpgrab_glue_schedular_move_collection(NNTPGrabGlue *obj, const char *collection_name, int new_position);
gboolean         nntpgrab_glue_schedular_mark_task_optional(NNTPGrabGlue *obj, const char *collection_name, const char *subject, ngboolean is_optional);
NGList          *nntpgrab_glue_plugins_get_avail_plugins(NNTPGrabGlue *obj);
gboolean         nntpgrab_glue_plugins_get_plugin_info(NNTPGrabGlue *obj, const char *plugin_name, NNTPGrabPluginInfo *plugin_info);
gboolean         nntpgrab_glue_plugins_load_plugin(NNTPGrabGlue *obj, const char *plugin_name, char **errmsg);
gboolean         nntpgrab_glue_plugins_unload_plugin(NNTPGrabGlue *obj, const char *plugin_name, char **errmsg);
gboolean         nntpgrab_glue_plugins_set_persistent(NNTPGrabGlue *obj, const char *plugin_name, ngboolean persistent);

#ifdef  __cplusplus
}
#endif

#endif /* _NNTPGRAB_GLUE_H_ */
